* Graphics for safety analysis
  Figure 3.2, CDF plot of QTc data
  Peter Lane, 22 February 2012, based on more general code by Shi-Tao Yeh (GSK);

%let treat = rand;
%let subjid = pid;
%let response = base_chg;
%let title = %str(Cumulative distribution of maximum QTc change);
%let footnote = %str(Note: Increase <30 msec 'Normal', 30-60 msec 'Concern', >60 msec 'High');
%let ytitle = %str(Maximum change in QTc interval (msec));
%let xtitle = %str(Cumulative percent);
%let dir = %str(C:/book/Fig3_2/);
filename psname "&dir.Fig3_2.ps";

* Point to dataset location (sheet of the Excel workbook);
libname xl excel "&dir.qtcf1.xls" access=readonly;
%let data=xl.'Sheet1$'n;

* Pick out maximum for each subject;
data qtcf;
  set &data;
  by &subjid &response;
  rep = 1;
  if last.&subjid;
proc sort; 
  by &treat;

* Count number of subjects for each treatment;
proc summary data=qtcf;
  var rep;
  by &treat;
  output out=p1(drop=_type_ _freq_) sum=scount;

* Form text for key, including counts;
data p1;
  set p1 end=last;
  length newtrt $30.;
  trtn=_n_;
  newtrt=trim(left(&treat)) || ' (N=' || compress(scount) || ')';
run;
proc sort; 
  by &treat;
proc sort data=qtcf; 
  by &treat;

* Split data into new sets for treatment;
data data1 data2;
  merge qtcf p1;
  by &treat;
  pct=100/scount;
  if trtn=1 then output data1;
  else if trtn=2 then output data2;

* Compute cdf for each treatment;
proc sort data=data1;
  by &response;
data data1(keep=&response cdf &treat &subjid newtrt);
  set data1;
  cdf = pct * _n_;
proc sort data=data2;
  by &response;
data data2(keep=&response cdf &treat &subjid newtrt);
  set data2;
  cdf = pct * _n_;
run;

* Re-combine the separate sets;
data final;
  set data1 data2;
  keep &subjid &response cdf &treat newtrt;
proc sort; 
  by newtrt cdf;
run;

* Set details for graphics:
    colour PostScript with 600dpi
    PostScript hardware font similar to Arial
    aspect ratio 1.8 to avoid too steep a slope in the centre;
goptions reset=all device=ps600c rotate=landscape gsfname=psname gsfmode=replace
  ftext=hwpsl009 vorigin=0 horigin=0 vsize=5in hsize=9in;
* title1 justify=center height=10pt "&title";
footnote1 justify=left height=15pt "&footnote";
* Position key inside frame, in bottom RHS corner where the lines will not conflict;
legend1 position=(right bottom inside) frame across=1 mode=protect label=none
  value=(height=11pt);
* Draw step-plots, discriminated by cololur and line-style;
symbol1 interpol=steplj color=blue line=1 width=4;
symbol2 interpol=steplj color=red line=2 width=4;
axis1 label=(height=15pt angle=90 "&xtitle") width=2
  order=(0 to 100 by 10) value=(height=15pt);
axis2 label=(height=15pt "&ytitle") width=2 value=(height=15pt);

* Draw parallel CDF plots for each treatment;
proc gplot data=final;
  plot cdf*&response=newtrt / haxis=axis2 vaxis=axis1
    href=0 30 60 vref=0 10 20 30 40 50 60 70 80 90 100
    chref=green lhref=2 cvref=gray legend=legend1;
run;
quit;

