* Graphics for safety analysis
  Figure 3.4, Mean profile plot of QTc data
  Peter Lane, 22 February 2012, based on more general code by Shi-Tao Yeh (GSK)";

%let treat=rand;
%let subjid=pid;
%let response=base_chg;
%let title=%str(QTc interval mean change from baseline);
%let footnote=%str(Note: LOCF is last observation carried forward);
%let ytitle=%str(Mean change (msec) and 95% CI);
%let xtitle=%str(Week);
%let dir=%str(C:/book/Fig3_4/);
filename psname "&dir.Fig3_4.ps";

* Point to dataset location (sheet of the Excel workbook);
libname xl excel "&dir.qtcf2.xls" access=readonly;
%let data=xl.'Sheet1$'n;

* Form dataset ECG with data at each post-baseline visit;
data ecg;
  set &data;
  if 10 => vis_id > 3 and &response^=.;
proc sort;
  by vis_id &subjid &response;
data ecg;
  set ecg;
  by vis_id &subjid &response;
  if last.&subjid;

* Form dataset BASE with baseline data;
data base;
  set &data;
  &response=0;
  if visit='BASELINE';
  keep &subjid &treat vis_id visit &response value;
run;
proc sort;
  by &subjid vis_id;
data base;
  set base;
  by &subjid vis_id;
  if last.&subjid;
run;

* Form dataset ENDPOINT with LOCF outcome;
proc sort data=ecg;
  by &subjid vis_id;
data endpoint;
  set ecg;
  by &subjid vis_id;
  vis_id=11;
  if last.&subjid;
proc sort;
  by &subjid;
data ba(keep=&subjid);
  set base;
proc sort;
  by &subjid;
data endpoint;
  merge ba(in=a) endpoint(in=b);;
  by &subjid;
  if a and b;
proc sort;
  by &treat;

* Form mean and CI for margin (LOCF);
proc summary data=endpoint;
  var &response;
  by &treat;
  id vis_id;
  output out=p2 n=n mean=mean lclm=lclm uclm=uclm max=max;

* Combine ECG & BAE and form means and CIs for each visit;
data ecg;
  set ecg base;
proc sort data=ecg;
  by &treat vis_id;
proc summary data=ecg;
  var &response;
  by &treat vis_id;
  id visit;
  output out=p1 n=n mean=mean lclm=lclm uclm=uclm max=max;

* Merge summary stats into FINAL dataset;
data final;
  set p1 p2;
proc sort;
  by &treat vis_id;
data final;
  set final;
  mean=round(mean,.01);
  lclm=round(lclm,.01);
  uclm=round(uclm,.01);
  * Set actual time of visit in variable VS2;
  if vis_id=11 then vs2=28;
  if visit = 'BASELINE' then vs2=0;
  else if visit='VISIT 3' then vs2=1;
  else if visit='VISIT 4' then vs2=2;
  else if visit='VISIT 5' then vs2=4;
  else if visit='VISIT 6' then vs2=8;
  else if visit='VISIT 7' then vs2=12;
  else if visit='VISIT 8' then vs2=16;
  else if visit='VISIT 9' then vs2=20;
  else if visit='VISIT 10' then vs2=24;
  * Offset the plotting to avoid over-writing;
  if &treat='Drug B' then vs2 = vs2+0.2;
  else vs2 = vs2-0.2;
  if visit='BASELINE' then do;
    lclm=.;
    uclm=.;
  end;
run;
data final;
  set final;
  call symput('n' ||compress(_n_), compress(put(n, 3.0)));
run;

* Set up annotation dataset to draw CIs;
data anno;
  set final;
  length function $8. text $1.;
  xsys='2'; ysys='2';
  line=1;
  size=5;
  if vs2 < 0.8 then delete;
  if &treat='Drug A' then color = 'blue';
  else color = 'red';
  x = vs2-0.025; * fix to avoid unexpected offset;
  y = lclm;
  function = 'MOVE'; output;
  function = 'DRAW'; output;
  y = uclm;
  function = 'DRAW'; output;
  x = vs2+0.27;
  y = uclm;
  function = 'MOVE'; output;
  x = vs2-0.27;
  function = 'DRAW'; output;
  x = vs2+0.27;
  y = lclm;
  function = 'MOVE'; output;
  x = vs2-0.27;
  function = 'DRAW'; output;

* Set up second annotation dataset to draw symbols for margin;
data anno1;
  set p2;
  length function style $8. text $1.;
  xsys='2'; ysys='2';
  if &treat='Drug A' then do;
    color='blue';
    vs2=27.8;
    x=vs2;
    y=mean;
    function='MOVE'; output;
    style='orfont';
    text='W';
    size=1;
    function='SYMBOL'; output;
  end;
  else do;
    color='red';
    vs2=28.2;
    x=vs2;
    y=mean;
    function='MOVE'; output;
    style='orfont';
    text='C';
    size=1;
    function='SYMBOL'; output;
  end;

* Combine annotation datasets;
data anno;
  set anno anno1;
run;

* Avoid joining last visit with marginal values;
data final;
  set final;
  length drug $6.;
  if vs2 > 25 then delete;

* Set details for graphics:
    colour PostScript with 600dpi
    PostScript hardware font similar to Arial
    aspect ratio 1.5;
goptions reset=all device=ps600c gsfname=psname rotate=landscape gsfmode=replace
  ftext=hwpsl009 vorigin=1in horigin=1in hsize=9in vsize=6in;
*title1 j=c h=15pt "&title";
footnote4 h=12pt j=l  "&footnote";
symbol1 i=j f=orfont v='W' h=9pt l=1 c=blue width=10;
symbol2 i=j f=orfont v='C' h=9pt l=1 c=red width=10;
axis1 order=(-2 to 28 by 2) offset=(0,5) major=(h=5pt) minor=none label=(h=12pt "&xtitle") width=1 
  value=(h=12pt tick=1 ' ' tick=5 ' ' tick=7 ' ' tick=9 ' ' tick=11 ' ' tick=13 ' ' 
  tick=15 ' ' tick=16 'LOCF');
axis2 order=(-6 to 3 by 1) width=1 value=(h=12pt) label=(h=12pt a=90 "&ytitle");
legend1 label=none value=(h=12pt) shape=symbol(0.1,0.8) position=(outside bottom left)
  mode=share cborder=white; *need border to avoid first symbol getting truncated;

* Draw mean lines with symbols at successive visits;
proc gplot data=final;
  plot mean * vs2 = &treat / vaxis=axis2 haxis=axis1 annotate=anno
  vref=0 lvref=2 href=26 lhref=0 cvref=green chref=black legend=legend1;

* Add lower margin with numbers of patients at each visit;
%let a = 10;
%let aa = 9.6;
%let b = 6.5;
%let c = 3.5;
%let d = 25.2;
%let h = 11pt;
note move=(10.5,&a)pct h=12pt 'Number of subjects at visit';
note move=(0,&b)pct h=&h c=blue 'Drug A';
note move=(0,&c)pct h=&h c=red 'Drug B';
note move=(10.5,&b)pct h=&h c=blue "&n1";
note move=(10.5,&c)pct h=&h c=red "&n11";
note move=(13.8,&b)pct h=&h c=blue "&n2";
note move=(13.8,&c)pct h=&h c=red "&n12";
note move=(17,&b)pct h=&h c=blue "&n3";
note move=(17,&c)pct h=&h c=red "&n13";
note move=(22.5,&b)pct h=&h c=blue "&n4";
note move=(22.5,&c)pct h=&h c=red "&n14";
note move=(34,&b)pct h=&h c=blue "&n5";
note move=(34,&c)pct h=&h c=red "&n15";
note move=(46,&b)pct h=&h c=blue "&n6";
note move=(46,&c)pct h=&h c=red "&n16";
note move=(58,&b)pct h=&h c=blue "&n7";
note move=(58,&c)pct h=&h c=red "&n17";
note move=(69.2,&b)pct h=&h c=blue "&n8";
note move=(69.2,&c)pct h=&h c=red "&n18";
note move=(81.4,&b)pct h=&h c=blue "&n9";
note move=(81.4,&c)pct h=&h c=red "&n19";
note move=(93,&b)pct h=&h c=blue "&n10";
note move=(93,&c)pct h=&h c=red "&n20";
run;
quit;
